
//         Install Scripts as plug-in , TEMPLATE
//                         by
//  B.Schaffer (FELMI/ZFE) D.R. G. Mitchell (ANSTO Materials)
//
//   Acknowledgements: Werner Grogger (FELMI/ZFE)
//                     Vincent Hou (Micron Tech.)
//
//   Template version: 3.0


/******* Constants of the package ********/
/*    Please modify this to your needs     */
/*********************************************/

string PackageName      = "My_Package"	// This is the filename of the plug-in
number PackageLevel   = 0			// This defines the loading priority of
					// this plug-in compared to others. 
string MenuName            = "Custom"		// This is the default name of the menu
number PackageVersion = 20071121		// This number gives the file version of 
					// the plug-in. It is convenient to use 
					// date related numbers of type YYYYMMDD.
string folder				// The path to the script containing folder
					// including the last backslash (\)
number lineCount      = 0			// Variable to count number of installed "lines" in the menu.

string GlobalPath = "C:\\"			// This is the path to the network-storage of  all new script-packages.

// Please note that these constants are declared as global variables which 
// are therefore also valid within the functions below!


/********* Auxillary Functions  **********/
/*       No modification necessary        */
/*******************************************/
number AddS2P(string ScriptName, string CommandName, string Menu, string SubMenu)
{
  // This function installs a script as a menu command
  result(" Installing <"+ScriptName+">\n\t")
  try 
   {
   AddScriptFileToPackage(folder+ScriptName,PackageName,PackageLevel,CommandName,Menu,SubMenu,0)
   }
  catch
   {
   beep()
   result(" => Error while installing as a menu command ("+Menu+"-"+SubMenu+"-"+CommandName+").\n")	
   return -1
   }
  result(" => Installed as a menu command ("+Menu+"-"+SubMenu+"-"+CommandName+").\n")	
  return 1
}


number AddLib2P(string ScriptName)
  {
  // This function installs a script as a library
  result(" Installing <"+ScriptName+">\n\t")
  try 
    {
    AddScriptFileToPackage(folder+ScriptName,PackageName,PackageLevel,ScriptName,"","",1)
    }
  catch
   {
   beep()
   result(" => Error while installing as a library.\n")	
   return -1
   }
  result(" => Installed as a library.\n")	
  return 1
}


number AddLine2P(string Menu, string SubMenu)
{
  // This function intalls a seperation line in a menu
  // Note, that it is necessary to install each line under a different 
  // "position" with regard to menu/submenu/command-name. That is the 
  // reason, why the line-count variable is needed.
  LineCount++
  result(" Installing a line\n\t")
  AddScriptToPackage("",PackageName,PackageLevel,"-"+menu+"."+LineCount, menu, submenu, 0)
  result(" => Installed.\n")
  return 1
}


number AddScriptString2P(string ScriptString, string CommandName, string Menu, string SubMenu, number AsLibrary)
{
  // This function installs a script-string (a script contained in a string variable) as a menu or library command.
  // It installs using the command "AddScriptToPackage" which installs a script (as a string) directly.
  result(" Installing ScriptString ("+CommandName+")\n\t")
  try 
    {
    AddScriptTopackage(ScriptString,PackageName,PackageLevel,CommandName,Menu,SubMenu,AsLibrary)
    }
  catch
   {
   beep()
   result(" => Error while installing.\n")	
   return -1
   }
  result(" => Installed as ")
  If (!AsLibrary) result("a menu command ("+Menu+"-"+SubMenu+"-"+CommandName+").\n")	
  else            result("a library command ("+CommandName+").\n")	
  return 1
}

string ReplaceBackSlashByDouble(string IN)
{
  // This function replaces all backslashes in a string by double-backslashes
  // This is needed whenever a string containing real backslashes should be used
  // hard coded in a script, as for example when installing a ScriptLine
  string out=""
  number pos=find(IN,"\\")
  While (pos!=-1)
    {
    out += left(IN,pos)+"\\\\"
    IN = mid(IN,pos+1,len(IN)-pos-1)
    pos=find(IN,"\\")
   }
  out+=IN
  return out
}

string ScriptGetVersion()
{
  // This function puts together the script for returning the package version.	
  // The whole script is contained in one string.
  string out
  out = "number "+PackageName+"_version() return "+Format(PackageVersion,"%8.f")
  return out
  }

string ScriptCompareVersion()
{
  // This function puts together the script for comparing the package version
  // with the net version. The whole script is contained in one string.
  // This script looks kind of strange because all backslashes in paths have 
  // to be replaced by double-backslashes if used as hardcoded string in a script
 string out
  out = "void "+PackageName+"_check()\n"
  out += "{\n"
  out += "number instV = "+Format(PackageVersion,"%8.f")+"\n"
  out += "number currV\n"
  out += "result(\"\\n ----------------------------------\")\n"
  out += "result(\"\\n   "+Packagename+"\")\n"
  out += "result(\"\\n ----------------------------------\")\n"
  out += "result(\"\\n  installed version: \"+Format(instV,\"%8.f\")+\"\")\n"
  out += "try \n{\ncurrV = ExecuteScriptFile(\""+ReplaceBackSlashByDouble(GlobalPath+PackageName)+"_v.s"+"\")\n}\n"
  out += "catch \n{\n Throw(\"Error running '"+ReplaceBackSlashByDouble(GlobalPath+PackageName)+"_v.s' \\nPossible reason: file not found.\")\n}\n\n"
  out += "result(\"\\n     newest version: \"+Format(currV,\"%8.f\")+\"\")\n"
  out += "result(\"\\n ----------------------------------\")\n"
  out += "If (currV>instV) OKDialog(\"A newer version of the plug-in is available!\\nnewest version: \"+Format(currV,\"%8.f\"))\n"
  out += "If (currV==instV) OKDialog(\"The installed version of the plug-in is up to date!\\ncurrent version: \"+Format(currV,\"%8.f\"))\n"
  out += "If (currV<instV) OKDialog(\"For some reason the installed version of the plug-in is newer than the one on the network!\\nnetwork version: \"+Format(currV,\"%8.f\"))\n"
  out += "}\n"
  return out
}

string ScriptOutputPackageInfo()
{
  // This function puts together the script for printing the package info stored
  // as a textfile on the network into the results window. 
  // The whole script is contained in one string.
  // This script looks kind of strange because all backslashes in paths have 
  // to be replaced by double-backslashes if used as hard coded string in a script
  string out
  out = "void "+PackageName+"_info()\n"
  out += "{\n"
  out += "number fileID\n"
  out += "string line\n"
  out += "string path = \""+ReplaceBackSlashByDouble(GlobalPath+PackageName)+"_info.txt\"\n"
  out += "if (!DoesFileExist(path))\n  {\n  OKDialog(\"No information text found.\")\n  exit(0)\n  }\n"
  out += "fileID = OpenFileForReading(path)\n"
  out += "result(\"\\n\\n Current information on package <"+PackageName+">\\n................................\\n\")\n"
  out += "While (ReadFileLine(fileID,line)) result(line)\n"
  out += "CloseFile(fileID)\n"
  out += "result(\"\\n................................\")\n"
  out += "}\n"
  return out
}

string ScriptUpdateButton()
{
  // This function creates the script which is executed with the "Plug-in info" button
  string out
  out  =  "If (OptionDown()) "+PackageName+"_info()\n"
  out +=  "Else "+PackageName+"_check()\n"
  return out
}


/******* MAIN INSTALLATION SCRIPT ***/
/*    Please modify this to your needs       */
/***********************************************/
object	DLG1			// Object for the dialogues
documentWindow  DLGWindow	// DialogWindow
taggroup DLGtg,DLGitems		// Items for the dialogues
taggroup DropMenu,DropMenuItems	// DropMenu Items within the dialogues
number userlevel,language		// variables keeping the setting

result("\n Installing files for plug-in <"+PackageName+">\n")

// First get the base-folder
folder = GetApplicationDirectory(2,0)
If (!GetDirectoryDialog("Please select base folder.",folder,folder)) exit(-1)

// Build First Dialog (UserLevel)
DLGtg = DLGCreateDialog("Level",DLGitems)
DropMenu = DLGCreatePopup(DropMenuItems,1)
DropMenuItems.DLGAddPopupItemEntry("Novice user")
DropMenuItems.DLGAddPopupItemEntry("Experienced user")
DropMenuItems.DLGAddPopupItemEntry("Expert user")
DLGitems.DLGAddElement(DLGCreateLabel("Please select user level"))
DLGitems.DLGAddElement(DropMenu)

// Display First Dialog (UserLevel)
DLG1 = Alloc(UIframe).Init(DLGtg)
If (!DLG1.Pose()) exit(-1)

// Get the value (UserLevel)
DropMenu.DLGGetValue(userlevel)

// Build Sceond Dialog (Language)
DLGtg = DLGCreateDialog("Language",DLGitems)
DropMenu = DLGCreatePopup(DropMenuItems,1)
DropMenuItems.DLGAddPopupItemEntry("English")
DropMenuItems.DLGAddPopupItemEntry("German")
DLGitems.DLGAddElement(DLGCreateLabel("Please select command name language"))
DLGitems.DLGAddElement(DropMenu)

// Display Second Dialog (Language)
DLG1 = Alloc(UIframe).Init(DLGtg)
IF (!DLG1.Pose()) exit(-1)

// Get the value (Language)
DropMenu.DLGGetValue(language)

// Now install all your script. Modify this section manually to your needs.
AddLib2P("MyLibrary.s")
// The different "languages" lead to different installation trees
If (language==1)
	{
	If (UserLevel>0) AddS2P("script1.s","NOVICE Command",MenuName,"")
	If (UserLevel>1) AddS2P("script1.s","EXPERIENCED Command",MenuName,"")
	If (UserLevel>2) AddS2P("script1.s","EXPERT Command",MenuName,"")
	}
If (language==2)
	{
	If (UserLevel>0) AddS2P("script1.s","Anfnger",MenuName,"")
	If (UserLevel>1) AddS2P("script1.s","Benutzer",MenuName,"")
	If (UserLevel>2) AddS2P("script1.s","Experte",MenuName,"")
	}

// Install the functions necessary for version checking as library
AddScriptString2P(ScriptCompareVersion(),PackageName+"_check","","",1)
AddScriptString2P(ScriptOutputPackageInfo(),PackageName+"_info","","",1)
AddScriptString2P(ScriptUpdateButton(),"Plug-in info",MenuName,"",0)
